"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPathKey = getPathKey;
exports.VERSION_COLOR_SCHEME = exports.ENV_PATH_KEY = exports.SINGLE_INSTANCE_FILENAME = exports.SINGLE_INSTANCE_PORT = exports.DEFAULT_INDENT = exports.NPM_SHRINKWRAP_FILENAME = exports.NPM_LOCK_FILENAME = exports.CLEAN_FILENAME = exports.TARBALL_FILENAME = exports.METADATA_FILENAME = exports.LOCKFILE_FILENAME = exports.INTEGRITY_FILENAME = exports.META_FOLDER = exports.FALLBACK_GLOBAL_PREFIX = exports.POSIX_GLOBAL_PREFIX = exports.PNP_FILENAME = exports.NODE_PACKAGE_JSON = exports.NODE_MODULES_FOLDER = exports.YARN_BIN_PATH = exports.NODE_BIN_PATH = exports.GLOBAL_MODULE_DIRECTORY = exports.LINK_REGISTRY_DIRECTORY = exports.DATA_DIRECTORY = exports.CONFIG_DIRECTORY = exports.PREFERRED_MODULE_CACHE_DIRECTORIES = exports.REQUIRED_PACKAGE_KEYS = exports.CHILD_CONCURRENCY = exports.NETWORK_TIMEOUT = exports.NETWORK_CONCURRENCY = exports.LOCKFILE_VERSION = exports.CACHE_VERSION = exports.SELF_UPDATE_VERSION_URL = exports.YARN_INSTALLER_MSI = exports.YARN_INSTALLER_SH = exports.YARN_DOCS = exports.NPM_REGISTRY_RE = exports.YARN_REGISTRY = exports.SUPPORTED_NODE_VERSIONS = exports.MANIFEST_FIELDS = exports.RESOLUTIONS = exports.OWNED_DEPENDENCY_TYPES = exports.DEPENDENCY_TYPES = void 0;

var os = require('os');

var path = require('path');

var userHome = require('./util/user-home-dir')["default"];

var _require = require('./util/user-dirs'),
    getCacheDir = _require.getCacheDir,
    getConfigDir = _require.getConfigDir,
    getDataDir = _require.getDataDir;

var isWebpackBundle = typeof __webpack_require__ !== "undefined";
var DEPENDENCY_TYPES = ['devDependencies', 'dependencies', 'optionalDependencies', 'peerDependencies'];
exports.DEPENDENCY_TYPES = DEPENDENCY_TYPES;
var OWNED_DEPENDENCY_TYPES = ['devDependencies', 'dependencies', 'optionalDependencies'];
exports.OWNED_DEPENDENCY_TYPES = OWNED_DEPENDENCY_TYPES;
var RESOLUTIONS = 'resolutions';
exports.RESOLUTIONS = RESOLUTIONS;
var MANIFEST_FIELDS = [RESOLUTIONS].concat(DEPENDENCY_TYPES);
exports.MANIFEST_FIELDS = MANIFEST_FIELDS;
var SUPPORTED_NODE_VERSIONS = '^4.8.0 || ^5.7.0 || ^6.2.2 || >=8.0.0';
exports.SUPPORTED_NODE_VERSIONS = SUPPORTED_NODE_VERSIONS;
var YARN_REGISTRY = 'https://registry.yarnpkg.com';
exports.YARN_REGISTRY = YARN_REGISTRY;
var NPM_REGISTRY_RE = /https?:\/\/registry\.npmjs\.org/g;
exports.NPM_REGISTRY_RE = NPM_REGISTRY_RE;
var YARN_DOCS = 'https://yarnpkg.com/en/docs/cli/';
exports.YARN_DOCS = YARN_DOCS;
var YARN_INSTALLER_SH = 'https://yarnpkg.com/install.sh';
exports.YARN_INSTALLER_SH = YARN_INSTALLER_SH;
var YARN_INSTALLER_MSI = 'https://yarnpkg.com/latest.msi';
exports.YARN_INSTALLER_MSI = YARN_INSTALLER_MSI;
var SELF_UPDATE_VERSION_URL = 'https://yarnpkg.com/latest-version'; // cache version, bump whenever we make backwards incompatible changes

exports.SELF_UPDATE_VERSION_URL = SELF_UPDATE_VERSION_URL;
var CACHE_VERSION = 6; // lockfile version, bump whenever we make backwards incompatible changes

exports.CACHE_VERSION = CACHE_VERSION;
var LOCKFILE_VERSION = 1; // max amount of network requests to perform concurrently

exports.LOCKFILE_VERSION = LOCKFILE_VERSION;
var NETWORK_CONCURRENCY = 8; // HTTP timeout used when downloading packages

exports.NETWORK_CONCURRENCY = NETWORK_CONCURRENCY;
var NETWORK_TIMEOUT = 30 * 1000; // in milliseconds
// max amount of child processes to execute concurrently

exports.NETWORK_TIMEOUT = NETWORK_TIMEOUT;
var CHILD_CONCURRENCY = 5;
exports.CHILD_CONCURRENCY = CHILD_CONCURRENCY;
var REQUIRED_PACKAGE_KEYS = ['name', 'version', '_uid'];
exports.REQUIRED_PACKAGE_KEYS = REQUIRED_PACKAGE_KEYS;

function getPreferredCacheDirectories() {
  var preferredCacheDirectories = [getCacheDir()];

  if (process.getuid) {
    // $FlowFixMe: process.getuid exists, dammit
    preferredCacheDirectories.push(path.join(os.tmpdir(), ".yarn-cache-".concat(process.getuid())));
  }

  preferredCacheDirectories.push(path.join(os.tmpdir(), ".yarn-cache"));
  return preferredCacheDirectories;
}

var PREFERRED_MODULE_CACHE_DIRECTORIES = getPreferredCacheDirectories();
exports.PREFERRED_MODULE_CACHE_DIRECTORIES = PREFERRED_MODULE_CACHE_DIRECTORIES;
var CONFIG_DIRECTORY = getConfigDir();
exports.CONFIG_DIRECTORY = CONFIG_DIRECTORY;
var DATA_DIRECTORY = getDataDir();
exports.DATA_DIRECTORY = DATA_DIRECTORY;
var LINK_REGISTRY_DIRECTORY = path.join(DATA_DIRECTORY, 'link');
exports.LINK_REGISTRY_DIRECTORY = LINK_REGISTRY_DIRECTORY;
var GLOBAL_MODULE_DIRECTORY = path.join(DATA_DIRECTORY, 'global');
exports.GLOBAL_MODULE_DIRECTORY = GLOBAL_MODULE_DIRECTORY;
var NODE_BIN_PATH = process.execPath;
exports.NODE_BIN_PATH = NODE_BIN_PATH;
var YARN_BIN_PATH = getYarnBinPath(); // Webpack needs to be configured with node.__dirname/__filename = false

exports.YARN_BIN_PATH = YARN_BIN_PATH;

function getYarnBinPath() {
  if (isWebpackBundle) {
    return __filename;
  } else {
    return path.join(__dirname, '..', 'bin', 'yarn.js');
  }
}

var NODE_MODULES_FOLDER = 'node_modules';
exports.NODE_MODULES_FOLDER = NODE_MODULES_FOLDER;
var NODE_PACKAGE_JSON = 'package.json';
exports.NODE_PACKAGE_JSON = NODE_PACKAGE_JSON;
var PNP_FILENAME = '.pnp.js';
exports.PNP_FILENAME = PNP_FILENAME;
var POSIX_GLOBAL_PREFIX = "".concat(process.env.DESTDIR || '', "/usr/local");
exports.POSIX_GLOBAL_PREFIX = POSIX_GLOBAL_PREFIX;
var FALLBACK_GLOBAL_PREFIX = path.join(userHome, '.yarn');
exports.FALLBACK_GLOBAL_PREFIX = FALLBACK_GLOBAL_PREFIX;
var META_FOLDER = '.yarn-meta';
exports.META_FOLDER = META_FOLDER;
var INTEGRITY_FILENAME = '.yarn-integrity';
exports.INTEGRITY_FILENAME = INTEGRITY_FILENAME;
var LOCKFILE_FILENAME = 'yarn.lock';
exports.LOCKFILE_FILENAME = LOCKFILE_FILENAME;
var METADATA_FILENAME = '.yarn-metadata.json';
exports.METADATA_FILENAME = METADATA_FILENAME;
var TARBALL_FILENAME = '.yarn-tarball.tgz';
exports.TARBALL_FILENAME = TARBALL_FILENAME;
var CLEAN_FILENAME = '.yarnclean';
exports.CLEAN_FILENAME = CLEAN_FILENAME;
var NPM_LOCK_FILENAME = 'package-lock.json';
exports.NPM_LOCK_FILENAME = NPM_LOCK_FILENAME;
var NPM_SHRINKWRAP_FILENAME = 'npm-shrinkwrap.json';
exports.NPM_SHRINKWRAP_FILENAME = NPM_SHRINKWRAP_FILENAME;
var DEFAULT_INDENT = '  ';
exports.DEFAULT_INDENT = DEFAULT_INDENT;
var SINGLE_INSTANCE_PORT = 31997;
exports.SINGLE_INSTANCE_PORT = SINGLE_INSTANCE_PORT;
var SINGLE_INSTANCE_FILENAME = '.yarn-single-instance';
exports.SINGLE_INSTANCE_FILENAME = SINGLE_INSTANCE_FILENAME;
var ENV_PATH_KEY = getPathKey(process.platform, process.env);
exports.ENV_PATH_KEY = ENV_PATH_KEY;

function getPathKey(platform, env) {
  var pathKey = 'PATH'; // windows calls its path "Path" usually, but this is not guaranteed.

  if (platform === 'win32') {
    pathKey = 'Path';

    for (var _key in env) {
      if (_key.toLowerCase() === 'path') {
        pathKey = _key;
      }
    }
  }

  return pathKey;
}

var VERSION_COLOR_SCHEME = {
  major: 'red',
  premajor: 'red',
  minor: 'yellow',
  preminor: 'yellow',
  patch: 'green',
  prepatch: 'green',
  prerelease: 'red',
  unchanged: 'white',
  unknown: 'red'
};
exports.VERSION_COLOR_SCHEME = VERSION_COLOR_SCHEME;
//# sourceMappingURL=constants.js.map
