"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.forkp = forkp;
exports.spawnp = spawnp;
exports.forwardSignalToSpawnedProcesses = forwardSignalToSpawnedProcesses;
exports.spawn = spawn;
exports.exec = exports.queue = void 0;

function constants() {
  var data = _interopRequireWildcard(require("../constants.js"));

  constants = function constants() {
    return data;
  };

  return data;
}

function _blockingQueue() {
  var data = _interopRequireDefault(require("./blocking-queue.js"));

  _blockingQueue = function _blockingQueue() {
    return data;
  };

  return data;
}

function _errors() {
  var data = require("../errors.js");

  _errors = function _errors() {
    return data;
  };

  return data;
}

function _promise() {
  var data = require("./promise.js");

  _promise = function _promise() {
    return data;
  };

  return data;
}

/* global child_process$spawnOpts */
var child = require('child_process');

var queue = new (_blockingQueue()["default"])('child', constants().CHILD_CONCURRENCY); // TODO: this uid check is kinda whack

exports.queue = queue;
var uid = 0;
var exec = (0, _promise().promisify)(child.exec);
exports.exec = exec;

function forkp(program, args, opts) {
  return new Promise(function (resolve, reject) {
    var proc = child.fork(program, args, opts);
    proc.on('error', function (error) {
      reject(error);
    });
    proc.on('close', function (exitCode) {
      resolve(exitCode);
    });
  });
}

function spawnp(program, args, opts) {
  return new Promise(function (resolve, reject) {
    var proc = child.spawn(program, args, opts);
    proc.on('error', function (error) {
      reject(error);
    });
    proc.on('close', function (exitCode) {
      resolve(exitCode);
    });
  });
}

var spawnedProcesses = {};

function forwardSignalToSpawnedProcesses(signal) {
  for (var _i = 0, _Object$keys = Object.keys(spawnedProcesses); _i < _Object$keys.length; _i++) {
    var key = _Object$keys[_i];
    spawnedProcesses[key].kill(signal);
  }
}

function spawn(program, args) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var onData = arguments.length > 3 ? arguments[3] : undefined;
  var key = opts.cwd || String(++uid);
  return queue.push(key, function () {
    return new Promise(function (resolve, reject) {
      var proc = child.spawn(program, args, opts);
      spawnedProcesses[key] = proc;
      var processingDone = false;
      var processClosed = false;
      var err = null;
      var stdout = '';
      proc.on('error', function (err) {
        if (err.code === 'ENOENT') {
          reject(new (_errors().ProcessSpawnError)("Couldn't find the binary ".concat(program), err.code, program));
        } else {
          reject(err);
        }
      });

      function updateStdout(chunk) {
        stdout += chunk;

        if (onData) {
          onData(chunk);
        }
      }

      function finish() {
        delete spawnedProcesses[key];

        if (err) {
          reject(err);
        } else {
          resolve(stdout.trim());
        }
      }

      if (typeof opts.process === 'function') {
        opts.process(proc, updateStdout, reject, function () {
          if (processClosed) {
            finish();
          } else {
            processingDone = true;
          }
        });
      } else {
        if (proc.stderr) {
          proc.stderr.on('data', updateStdout);
        }

        if (proc.stdout) {
          proc.stdout.on('data', updateStdout);
        }

        processingDone = true;
      }

      proc.on('close', function (code, signal) {
        if (signal || code >= 1) {
          err = new (_errors().ProcessTermError)(['Command failed.', signal ? "Exit signal: ".concat(signal) : "Exit code: ".concat(code), "Command: ".concat(program), "Arguments: ".concat(args.join(' ')), "Directory: ".concat(opts.cwd || process.cwd()), "Output:\n".concat(stdout.trim())].join('\n'));
          err.EXIT_SIGNAL = signal;
          err.EXIT_CODE = code;
        }

        if (processingDone || err) {
          finish();
        } else {
          processClosed = true;
        }
      });
    });
  });
}
//# sourceMappingURL=child.js.map
