"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sortFilter = sortFilter;
exports.matchesFilter = matchesFilter;
exports.ignoreLinesToRegex = ignoreLinesToRegex;
exports.filterOverridenGitignores = filterOverridenGitignores;

function _toConsumableArray2() {
  var data = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

  _toConsumableArray2 = function _toConsumableArray2() {
    return data;
  };

  return data;
}

function _misc() {
  var data = require("./misc.js");

  _misc = function _misc() {
    return data;
  };

  return data;
}

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var mm = require('micromatch');

var path = require('path');

var WHITESPACE_RE = /^\s+$/;

function sortFilter(files, filters) {
  var keepFiles = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new Set();
  var possibleKeepFiles = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : new Set();
  var ignoreFiles = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : new Set();

  var _iterator = _createForOfIteratorHelper(files),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var file = _step.value;
      var keep = false; // always keep a file if a ! pattern matches it

      var _iterator5 = _createForOfIteratorHelper(filters),
          _step5;

      try {
        for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
          var filter = _step5.value;

          if (filter.isNegation && matchesFilter(filter, file.basename, file.relative)) {
            keep = true;
            break;
          }
        } //

      } catch (err) {
        _iterator5.e(err);
      } finally {
        _iterator5.f();
      }

      if (keep) {
        keepFiles.add(file.relative);
        continue;
      } // otherwise don't keep it if a pattern matches it


      keep = true;

      var _iterator6 = _createForOfIteratorHelper(filters),
          _step6;

      try {
        for (_iterator6.s(); !(_step6 = _iterator6.n()).done;) {
          var _filter = _step6.value;

          if (!_filter.isNegation && matchesFilter(_filter, file.basename, file.relative)) {
            keep = false;
            break;
          }
        }
      } catch (err) {
        _iterator6.e(err);
      } finally {
        _iterator6.f();
      }

      if (keep) {
        possibleKeepFiles.add(file.relative);
      } else {
        ignoreFiles.add(file.relative);
      }
    } // exclude file

  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  var _iterator2 = _createForOfIteratorHelper(possibleKeepFiles),
      _step2;

  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var _file = _step2.value;
      var parts = path.dirname(_file).split(path.sep);

      while (parts.length) {
        var folder = parts.join(path.sep);

        if (ignoreFiles.has(folder)) {
          ignoreFiles.add(_file);
          break;
        }

        parts.pop();
      }
    } //

  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }

  var _iterator3 = _createForOfIteratorHelper(possibleKeepFiles),
      _step3;

  try {
    for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
      var _file2 = _step3.value;

      if (!ignoreFiles.has(_file2)) {
        keepFiles.add(_file2);
      }
    } //

  } catch (err) {
    _iterator3.e(err);
  } finally {
    _iterator3.f();
  }

  var _iterator4 = _createForOfIteratorHelper(keepFiles),
      _step4;

  try {
    for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
      var _file3 = _step4.value;

      var _parts = path.dirname(_file3).split(path.sep);

      while (_parts.length) {
        // deregister this folder from being ignored, any files inside
        // will still be marked as ignored
        ignoreFiles["delete"](_parts.join(path.sep));

        _parts.pop();
      }
    }
  } catch (err) {
    _iterator4.e(err);
  } finally {
    _iterator4.f();
  }

  return {
    ignoreFiles: ignoreFiles,
    keepFiles: keepFiles
  };
}

function matchesFilter(filter, basename, loc) {
  var filterByBasename = true;

  if (filter.base && filter.base !== '.') {
    loc = path.relative(filter.base, loc);
    filterByBasename = false;
  } // the micromatch regex expects unix path separators


  loc = loc.replace(/\\/g, '/');
  return filter.regex.test(loc) || filter.regex.test("/".concat(loc)) || filterByBasename && filter.regex.test(basename) || mm.isMatch(loc, filter.pattern);
}

function ignoreLinesToRegex(lines) {
  var base = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.';
  return lines // create regex
  .map(function (line) {
    // remove empty lines, comments, etc
    if (line === '' || line === '!' || line[0] === '#' || WHITESPACE_RE.test(line)) {
      return null;
    }

    var pattern = line;
    var isNegation = false; // hide the fact that it's a negation from minimatch since we'll handle this specifically
    // ourselves

    if (pattern[0] === '!') {
      isNegation = true;
      pattern = pattern.slice(1);
    } // remove trailing slash


    pattern = (0, _misc().removeSuffix)(pattern, '/');
    var regex = mm.makeRe(pattern.trim(), {
      dot: true,
      nocase: true
    });

    if (regex) {
      return {
        base: base,
        isNegation: isNegation,
        pattern: pattern,
        regex: regex
      };
    } else {
      return null;
    }
  }).filter(Boolean);
}

function filterOverridenGitignores(files) {
  var IGNORE_FILENAMES = ['.yarnignore', '.npmignore', '.gitignore'];
  var GITIGNORE_NAME = IGNORE_FILENAMES[2];
  return files.filter(function (file) {
    return IGNORE_FILENAMES.indexOf(file.basename) > -1;
  }).reduce(function (acc, file) {
    if (file.basename !== GITIGNORE_NAME) {
      return [].concat((0, _toConsumableArray2()["default"])(acc), [file]);
    } else {
      //don't include .gitignore if .npmignore or .yarnignore are present
      var dir = path.dirname(file.absolute);
      var higherPriorityIgnoreFilePaths = [path.join(dir, IGNORE_FILENAMES[0]), path.join(dir, IGNORE_FILENAMES[1])];
      var hasHigherPriorityFiles = files.find(function (file) {
        return higherPriorityIgnoreFilePaths.indexOf(path.normalize(file.absolute)) > -1;
      });

      if (!hasHigherPriorityFiles) {
        return [].concat((0, _toConsumableArray2()["default"])(acc), [file]);
      }
    }

    return acc;
  }, []);
}
//# sourceMappingURL=filter.js.map
